using System;
using System.Text;
using System.IO;
using System.Net;
using System.Collections.Generic;
using Terrasoft.Nui.ServiceModel.DataContract;
using Terrasoft.Core.Entities;
using System.Web.Script.Serialization;

namespace DataServiceBatchExample
{
    class Program
    {
        // Основной Uri приложения bpm'online. Необходимо заменить на пользовательский.
        private const string baseUri = @"http://example.bpmonline.com";
        // Строка запроса к методу Login сервиса AuthService.svc.
        private const string authServiceUri = baseUri + @"/ServiceModel/AuthService.svc/Login";
        // Строка пути запроса BatchQuery.
        private const string batchQueryUri = baseUri + @"/0/DataService/json/reply/BatchQuery";
        // Cookie аутентификации bpm'online.
        private static CookieContainer AuthCookie = new CookieContainer();

        // Метод выполняет аутентификацию пользователя.
        // Параметры:
        // userName - имя пользователя bpm'online,
        // userPassword - пароль пользователя bpm'online.
        private static bool TryLogin(string userName, string userPassword)
        {
            // Создание экземпляра запроса к сервису аутентификации.
            var authRequest = HttpWebRequest.Create(authServiceUri) as HttpWebRequest;
            // Определение метода HTTP-запроса.
            authRequest.Method = "POST";
            // Определение типа контента запроса.
            authRequest.ContentType = "application/json";
            // Включение использования cookie в запросе.
            authRequest.CookieContainer = AuthCookie;
            // Помещение в тело запроса учетной информации пользователя.
            using (var requesrStream = authRequest.GetRequestStream())
            {
                using (var writer = new StreamWriter(requesrStream))
                {
                    writer.Write(@"{
                        ""UserName"":""" + userName + @""",
                        ""UserPassword"":""" + userPassword + @"""
                    }");
                }
            }

            // Получение ответа от сервера. Если аутентификация проходит успешно, в свойство AuthCookie будут 
            // помещены cookie, которые могут быть использованы для последующих запросов.
            using (var response = (HttpWebResponse)authRequest.GetResponse())
            {
                if (AuthCookie.Count > 0)
                {
                    return true;
                }
            }
            return false;
        }

        // Главный метод приложения.
        static void Main(string[] args)
        {
            //Если выполнение аутентификации неудачно, приложение завершает работу.
            if (!TryLogin("Пользователь 1", "Пользователь 1"))
            {
                Console.WriteLine("Ошибка аутентификации!");
                return;
            };

            // Запрос на добавление данных.
            var insertQuery = new InsertQuery()
            {
                RootSchemaName = "Contact",
                ColumnValues = new ColumnValues()
                {
                    Items = new Dictionary<string, ColumnExpression>()
                    {
                        {
                            "Name",
                            new ColumnExpression()
                            {
                                ExpressionType = EntitySchemaQueryExpressionType.Parameter,
                                Parameter = new Parameter
                                {
                                    Value = "Петров Петр Петрович",
                                    DataValueType = DataValueType.Text
                                }
                            }
                        }
                    }
                }
            };

            // Запрос на обновление данных.
            var updateQuery = new UpdateQuery()
            {
                RootSchemaName = "Contact",
                ColumnValues = new ColumnValues()
                {
                    Items = new Dictionary<string, ColumnExpression>()
                    {
                        {
                            "Phone",
                            new ColumnExpression()
                            {
                                ExpressionType = EntitySchemaQueryExpressionType.Parameter,
                                Parameter = new Parameter()
                                {
                                    Value = "0123456789",
                                    DataValueType = DataValueType.Text
                                }
                            }
                        }
                    }
                },
                Filters = new Filters()
                {
                    FilterType = Terrasoft.Nui.ServiceModel.DataContract.FilterType.FilterGroup,
                    Items = new Dictionary<string, Filter>()
                    {
                        {
                            "FilterByName",
                            new Filter
                            {
                                FilterType = Terrasoft.Nui.ServiceModel.DataContract.FilterType.CompareFilter,
                                ComparisonType = FilterComparisonType.Equal,
                                LeftExpression = new BaseExpression()
                                {
                                    ExpressionType = EntitySchemaQueryExpressionType.SchemaColumn,
                                    ColumnPath = "Name"
                                },
                                RightExpression = new BaseExpression()
                                {
                                    ExpressionType = EntitySchemaQueryExpressionType.Parameter,
                                    Parameter = new Parameter()
                                    {
                                        DataValueType = DataValueType.Text,
                                        Value = "Петров Петр Петрович"
                                    }
                                }
                            }
                        }
                    }
                }
            };
            // Сериализация экземпляра класса запроса на добавление в JSON-строку.
            var jsonInsert = new JavaScriptSerializer().Serialize(insertQuery);
            // Вставка типа запроса в в JSON-строку.
            jsonInsert = jsonInsert.Insert(1, @"""__type"": ""Terrasoft.Nui.ServiceModel.DataContract.InsertQuery"",");
            // Сериализация экземпляра класса запроса на обновление в JSON-строку.
            var jsonUpdate = new JavaScriptSerializer().Serialize(updateQuery);
            // Вставка типа запроса в в JSON-строку.
            jsonUpdate = jsonUpdate.Insert(1, @"""__type"": ""Terrasoft.Nui.ServiceModel.DataContract.UpdateQuery"",");
            // Формирование пакетного запроса.
            var json = @"{""items"": [" + jsonInsert + "," + jsonUpdate + "]}";

            // Преобразование строки JSON-объекта в массив байтов.
            byte[] jsonArray = Encoding.UTF8.GetBytes(json);
            // Создание экземпляра HTTP-запроса.
            var batchRequest = HttpWebRequest.Create(batchQueryUri) as HttpWebRequest;
            // Определение метода запроса.
            batchRequest.Method = "POST";
            // Определение типа содержимого запроса.
            batchRequest.ContentType = "application/json";
            // Добавление полученных ранее аутентификационных cookie в запрос на получение данных.
            batchRequest.CookieContainer = AuthCookie;
            // Set the ContentLength property of the WebRequest.
            batchRequest.ContentLength = jsonArray.Length;

            // Помещение в содержимое запроса JSON-объекта.
            using (var requestStream = batchRequest.GetRequestStream())
            {
                requestStream.Write(jsonArray, 0, jsonArray.Length);
            }
            // Выполнение HTTP запроса и получение ответа от сервера.
            using (var response = (HttpWebResponse)batchRequest.GetResponse())
            {
                // Вывод ответа в консоль.
                using (StreamReader reader = new StreamReader(response.GetResponseStream()))
                {
                    Console.WriteLine(reader.ReadToEnd());
                }
            }
            // Задержка выполнения приложения.
            Console.ReadKey();
        }
    }
}
